
/*
 * Copyright (c) Developers of Binary Utility
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 *      <date>			        <changed by>		<summary of changes>
 * 19 December 2004  	    Jonee Ryan Ty		First documented history
 *
 */

#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

enum boolean {FALSE = 0, TRUE};

int error = FALSE;

int main(int argc, char* argv[]) {

        int aFileMissing = FALSE, exists(char *);
        void compare(char *, char *);


        if (argc != 3) {	// check for parameter count
		fprintf(stderr, "wrong parameter count\n");
		fprintf(stderr, "<file1> <file2>\n");
		error = TRUE;
	}


        if (argc > 1) {
                if (!exists(argv[1])) {
	        	fprintf(stderr, "file: %s does not exist\n", argv[1]);
		        error = TRUE;
                        aFileMissing = TRUE;
                }
        }

        if (argc > 2) {
                if (!exists(argv[2])) {
	        	fprintf(stderr, "file: %s does not exist\n", argv[2]);
		        error = TRUE;
                        aFileMissing = TRUE;
                }

                if ((strcmp(argv[1], argv[2]) == 0)) {
		        fprintf(stderr, "same file name?\n");
		        error = TRUE;
                }
        }

        if (argc < 3) aFileMissing = TRUE;

        if (!error) compare(argv[1], argv[2]);
}

/**
 * check whether the file exists
 */
int exists(const char *outFileName) {
	FILE *tmp;
	tmp = fopen(outFileName, "r");

	if (tmp != NULL) {
		fclose(tmp);
		return TRUE;
	}

	return FALSE;
}

void compare(char *f1, char *f2) {

        char *toString(int);
        int len, i = 0, hexOffset, iF1, iF2;

        struct stat st1, st2;
        FILE *inFile1, *inFile2;

        inFile1 = fopen(f1, "r");
	stat(f1, &st1);

        inFile2 = fopen(f2, "r");
	stat(f2, &st2);

        if (st1.st_size != st2.st_size) {
                printf("The two files do not have the same size.\n");
                len = (st1.st_size < st2.st_size)? st1.st_size: st2.st_size;

        } else len = st1.st_size;

        for (; i < len; i++) {
                fread(&iF1, 1, 1, inFile1);
                fread(&iF2, 1, 1, inFile2);

                if (iF1 != iF2) {
                        hexOffset = (int)strtol(toString(i), NULL, 16);
                        printf("The two files start differing at offset %d = 0x%d\n", i, hexOffset);
                        i = len;
                }
        }

        fclose(inFile1);
        fclose(inFile2);
}

char *toString(int i) {
        char *res;
        res = (char *)calloc(32, sizeof(char));
        sprintf(res, "%i", i);
        return res;
}

